import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
/**
 * Write a description of class SlideBar here.
 * 
 * @author (Moritz L.(Busch2207)) 
 * @version (14.05.2013)
 */
public class SlideBar extends Actor
{
    private static final GreenfootImage gi_sf_Slide=new GreenfootImage("Slide.gif");
    private int i_MaxValue,i_Value,i_SlideLocation,i_width;
    private Color col_slideBar,col_Number,col_rect,col_back;
    private boolean b_changedSinceLastTime=false,b_enabled=true;
    private ValueWriter vw_=new JustWriteIt();
    private java.util.List <SlideBarListener> list_sbl_= new java.util.ArrayList();

    public SlideBar(int width,int StartValue,int MaxValue,Color BarColor)
    {
        this(width,StartValue,MaxValue,BarColor,new Color(0,0,0,0),new Color(0,0,0,0),new Color(0,0,0,0));
    }

    public SlideBar(int width,int StartValue,int MaxValue,Color BarColor,Color RectColor)
    {
        this(width,StartValue,MaxValue,BarColor,new Color(0,0,0,0),RectColor,new Color(0,0,0,0));
    }

    public SlideBar(int width,int StartValue,int MaxValue,Color BarColor,Color NumberColor,Color RectColor)
    {
        this(width,StartValue,MaxValue,BarColor,NumberColor,RectColor,new Color(0,0,0,0));
    }

    public SlideBar(int width,int StartValue,int MaxValue,Color BarColor,Color NumberColor,Color RectColor,Color BackColor)
    {
        setImage(new GreenfootImage(width+gi_sf_Slide.getWidth(),gi_sf_Slide.getHeight()+(NumberColor.getAlpha()>0?25:6)));
        getImage().setFont(new Font("",Font.ROMAN_BASELINE,20));
        this.col_slideBar=BarColor;
        this.col_rect=RectColor;
        this.col_Number=NumberColor;
        this.col_back=BackColor;
        this.i_MaxValue=MaxValue;
        this.i_width=width;
        setValue(StartValue);
    }

    public void addListener(SlideBarListener sbl_)
    {
        list_sbl_.add(sbl_);
    }

    public void removeListener(SlideBarListener sbl_)
    {
        list_sbl_.remove(sbl_);
    }

    public void removeAllListener()
    {
        list_sbl_.clear();
    }

    public void setValueWriter(ValueWriter vw_)
    {
        this.vw_=vw_;
    }

    public ValueWriter getValueWriter()
    {
        return this.vw_;
    }

    /** enables / disable the SlideBar */
    public void enable(boolean b_enable)
    {
        this.b_enabled=b_enable;
        updateImage();
    }

    public void act() 
    {
        boolean change=b_enabled&&(Greenfoot.mouseDragged(this)||Greenfoot.mouseClicked(this));
        if(change)
        {
            b_changedSinceLastTime=true;
            setSlideLocation(Greenfoot.getMouseInfo().getX()-getX()+i_width/2-(i_width%2==0?1:0));
        }
    }

    private void setSlideLocation(int i_mouseLocation)
    {
        if(i_mouseLocation<0)
            i_mouseLocation=0;
        if(i_mouseLocation>i_width)
            i_mouseLocation=i_width;
        this.i_SlideLocation=i_mouseLocation;
        i_Value=(int)((double)i_mouseLocation/(double)i_width*(double)i_MaxValue);
        for(SlideBarListener sbl_:list_sbl_)
            sbl_.changed(i_Value);
        updateImage();
    }

    private void updateImage()
    {
        GreenfootImage i=getImage();
        i.clear();
        i.setColor(col_back);
        i.fill();
        i.setColor(col_rect);
        i.drawRect(0,0,i.getWidth()-1,i.getHeight()-1);
        i.setColor(col_slideBar);
        i.fillRect(gi_sf_Slide.getWidth()/2-1,0,3,11);
        i.fillRect(i_width+gi_sf_Slide.getWidth()/2-1,0,3,11);
        i.fillRect(i.getWidth()/2-1,0,3,11);
        i.fillRect(gi_sf_Slide.getWidth()/2,4,i_width,3);
        i.drawImage(gi_sf_Slide,i_SlideLocation,5);
        if(col_Number.getAlpha()!=0)
        {
            i.setColor(col_Number);
            i.drawString(vw_.getStringForValue(i_Value),0,20+gi_sf_Slide.getHeight());
        }
        if(!b_enabled)
        {
            //setImage(i=Picture.blackWhite(Picture.drawColorOverImage(new Color(100,100,100,100),i)));
            i.setColor(new Color(100,100,100,100));
            i.fill();
            i.setColor(new Color(255-col_back.getRed(),255-col_back.getGreen(),255-col_back.getBlue()));
            i.fillPolygon( // left top to right bottom
                new int[]{0,0,5,i.getWidth ()  ,i.getWidth (),i.getWidth ()-5},
                new int[]{5,0,0,i.getHeight()-5,i.getHeight(),i.getHeight()  },
                6);
            i.fillPolygon(// left bottom to right top
                new int[]{              0,            0,            5,i.getWidth(),i.getWidth(),i.getWidth ()-5},
                new int[]{i.getHeight()-5,i.getHeight(),i.getHeight(),           5,           0,              0},
                6);
        }
    }

    /** sets the Value, the SlideBar should show */
    public void setValue(int i_newValue)
    {
        this.i_Value=Math.max(0,Math.min(i_MaxValue,i_newValue));
        i_SlideLocation=(int)((double)i_Value/(double)i_MaxValue*(double)i_width);
        updateImage();
    }

    /** returns the current value, the SlideBar shows */
    public int getValue()
    {
        b_changedSinceLastTime=false;
        return i_Value;
    }

    /** returns, if the value, the SlideBar shows has changed */
    public boolean valueChanged()
    {
        return b_changedSinceLastTime;
    }

    public interface SlideBarListener
    {
        public void changed(int i_Value);
    }

    private class JustWriteIt implements ValueWriter
    {
        public String getStringForValue(int i_Value)
        {
            return i_Value+"";
        }
    }

    public interface ValueWriter
    {
        public String getStringForValue(int i_Value);
    }
}